#include <helianthus.h>
#include <stdlib.h>
#include <time.h>

#define MAX_THEME 8
int theme;
int help = TRUE;


const unsigned int themeColors[MAX_THEME][5] = {
  { 0x2e2f38ff, 0xe88638ff, 0xffd63eff, 0x8a221cff, 0xffb54bff },
  { 0x121212ff, 0x4d7b85ff, 0xc9f24bff, 0x23383dff, 0xe6f2daff },
  { 0x10222bff, 0xe2f0d6ff, 0xf6ffe0ff, 0x95ab63ff, 0xbdd684ff },
  { 0x000000ff, 0xb81111ff, 0x2d2b2aff, 0xffffffff, 0x561812ff },
  { 0x343f40ff, 0xbfac95ff, 0xf2d7b6ff, 0x736751ff, 0x8c3f3fff },
  { 0x333b3aff, 0xb8925aff, 0x61594dff, 0xb4bd51ff, 0x543b38ff },
  { 0x0d1114ff, 0xebbc5eff, 0x695f4cff, 0x102c2eff, 0xfffbb8ff },
  { 0x362c2aff, 0xfad9a0ff, 0x736859ff, 0x732420ff, 0xbf734cff }
};


Group balls;
Group border;
Sprite selected;
double selX, selY;


void createBall(int x, int y) {
  int radius = randomNumber(30, 60);
  Sprite s = createSprite(x, y, radius*2, radius*2);
  spriteSetColliderCircle(s, 0, 0, radius);
  spriteSetUserTag(s, rand());
  spriteSetBounciness(s, 0.8);
  spriteSetAccelerationY(s, 1500);
  groupAdd(balls, s);
}


void drawBall(Sprite s) {
  int r = rand();
  srand(spriteGetUserTag(s));
  int circlesCount = randomNumber(1, 5);
  for (int i = 0; i < circlesCount; i++) {
    int circleColor = themeColors[theme][randomNumber(1, 4)];
    noStroke();
    fill(circleColor);
    if (i > 0) {
      circle(spriteGetX(s), spriteGetY(s), ((1 - i/(double)circlesCount)*spriteGetWidth(s)/2)-randomNumber(0, 5));
    } else {
      circle(spriteGetX(s), spriteGetY(s), (1 - i/(double)circlesCount)*spriteGetWidth(s)/2);
    }
  }
  srand(r);
}


void updateBorder(double w, double h) {
  double b = 1000;
  double pos[4][4] = {
    {    -b/2, h/2, b, h + b*2 },
    { w + b/2, h/2, b, h + b*2 },
    { w/2,    -b/2, w + 2*b, b },
    { w/2, h + b/2, w + 2*b, b } };
  for(int i = 0; i < 4; ++i) {
    Sprite s = groupGet(border, i);
    spriteSetX(s, pos[i][0]);
    spriteSetY(s, pos[i][1]);
    spriteSetWidth(s, pos[i][2]);
    spriteSetHeight(s, pos[i][3]);
  }
}


void init() {
  theme = randomNumber(0, MAX_THEME - 1);
  balls = createGroup();
  background(themeColors[theme][0]);
  createBall(125, 125);

  border = createGroup();
  groupAdd(border, createSprite(0, 0, 1, 1));
  groupAdd(border, createSprite(0, 0, 1, 1));
  groupAdd(border, createSprite(0, 0, 1, 1));
  groupAdd(border, createSprite(0, 0, 1, 1));
  groupSetMassLevelEach(border, 100);
  groupSetBouncinessEach(border, 0.8);
}


void draw() {
  double t = windowGetFrameTime();

  double w = windowGetWidth();
  double h = windowGetHeight();
  updateBorder(w, h);

  double mx = mouseX(), my = mouseY();

  if (mouseWentDown("left")) {
    for(int i = 0; i < groupGetCount(balls); ++i) {
      Sprite s = groupGet(balls, i);
      if (spriteIsPointInside(s, mx, my)) {
        selected = s;
        spriteSetMassLevel(selected, 1);
        selX = spriteGetX(selected) - mx;
        selY = spriteGetY(selected) - my;
      }
    }
  }

  if (!mouseDown("left")) {
    if (selected) spriteSetMassLevel(selected, 0);
    selected = NULL;
  } else
  if (selected) {
    double px = spriteGetX(selected);
    double py = spriteGetY(selected);
    double nx = mx + selX;
    double ny = my + selY;
    double k = 0.4;
    spriteSetXY(selected, nx, ny);
    spriteSetVelocityXY(selected, (nx - px)/t*k, (ny - py)/t*k);
  }

  if (mouseWentDown("right")) {
    createBall(mx, my);
  }

  if (mouseWentDown("middle")) {
    theme = (theme + 1)%MAX_THEME;
    background(themeColors[theme][0]);
  }

  if (keyWentDown("space")) {
    groupDestroyEach(balls);
  }

  if (keyWentDown("f1")) {
    help = !help;
  }

  for(int i = 0; i < 10; ++i) {
    groupCollideBetween(balls);
    groupCollideGroup(border, balls);
  }

  for (int i = 0; i < groupGetCount(balls); i++) {
    drawBall(groupGet(balls, i));
  }

  if (help == TRUE) {
    noFill();
    stroke(themeColors[theme][1]);
    text(10, 10, "Press space to delete balls.\n"
                 "Press F1 to toggle this help.\n"
                 "Drag balls by left-click.\n"
                 "Right-click to create a ball.\n"
                 "Middle-click to toggle theme.\n" );
  }
}


int main() {
  windowSetTitle("Ball Pool");
  windowSetVariableFrameRate();
  windowSetResizable(TRUE);
  windowSetInit(&init);
  windowSetDraw(&draw);
  windowRun();
  return 0;
}
